"""jquery_alchemy is a module to handle generating 'rules' for the
jquery validation package from sqlalchemy columns.

It is designed to work with orm-based form generation modules,
such as wtforms_alchemy, deform (colander_alchemy) or formalchemy."""

from sqlalchemy import (
    Boolean,
    Float,
    inspect,
    Integer,
    Numeric,
    String,
    Text,
    Time,
    )

from sqlalchemy_utils import (
    EmailType,
    IPAddressType,
    URLType,
    )


def jquery_alchemy(cls):

    """Any sqlalchemy Column can have an 'info' dictionary.
    If this contains am explicit 'jquery_validate' sub-dict, use it.
    Otherwise, create the dict implicitly from Column information."""

    cls = inspect(cls)
    rules = {'rules': {}}
    for prop in cls.attrs:
        # Always a 1-item list?
        column = prop.columns[0]

        # Check for explicit rule definitions
        if 'jquery_validate' in column.info:
            rules['rules'][prop.key] = column.info['jquery_validate']
        else:
            # Create implicit rule definitions if no explicit ones set
            column.info['jquery_validate'] = {}

            # General rules which apply to all columns
            if not column.nullable:
                column.info['jquery_validate']['required'] = True

            if hasattr(column.type, 'length'):
                column.info['jquery_validate']['maxlength'] = column.type.length

            # Specific rules for particular column types
            if isinstance(column.type, EmailType):
                column.info['jquery_validate']['email'] = True

            elif isinstance(column.type, Integer):
                column.info['jquery_validate']['integer'] = True

            elif isinstance(column.type, IPAddressType):
                column.info['jquery_validate']['ipv4'] = True

            elif isinstance(column.type, Numeric):
                column.info['jquery_validate']['number'] = True

            elif isinstance(column.type, Time):
                column.info['jquery_validate']['time'] = True

            elif isinstance(column.type, URLType):
                column.info['jquery_validate']['url'] = True

            rules['rules'][prop.key] = column.info['jquery_validate']

    return rules



import setuptools
import subprocess

pkg_name = 'jquery_alchemy'
pkg_url = 'http://www.github.com/mrichar1/jquery_alchemy'
pkg_license = 'GNU AGPL v3'
pkg_description = 'A package which creates jquery_validation rules from sqlalchemy columns for use with web forms.'
pkg_author = 'Matthew Richardson.'
pkg_author_email = 'm.richardson@ed.ac.uk'

pkg_classifiers = [
            'Development Status :: 4 - Beta',
            'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 3',
            ]

install_requires = ['sqlalchemy', 'sqlalchemy_utils']


def call_git_describe():
    try:
        p = subprocess.Popen(['git', 'describe'],
                             stdout=subprocess.PIPE)
        return p.communicate()[0].split('\n')[0].strip()
    except Exception:
        return None

def read_release_version():
    try:
        with open("RELEASE-VERSION") as f:
            return f.readlines()[0].strip()
    except Exception:
        return None

def write_release_version(version):
    with open("RELEASE-VERSION", "w") as f:
        f.write("%s\n" % version)

def get_git_version():
    version = call_git_describe()
    release_version = read_release_version()
    if version is None:
        version = release_version

    if version is None:
        raise ValueError("Unable to determine the version number!")

    if version != release_version:
        write_release_version(version)

    return version


def main():

    setuptools.setup(
        name=pkg_name,
        version=get_git_version(),
        url=pkg_url,
        license=pkg_license,
        description=pkg_description,
        classifiers=pkg_classifiers,
        author=pkg_author,
	author_email=pkg_author_email,
        packages=setuptools.find_packages(),
        include_package_data=True,
        package_data={'': ['LICENSE', 'RELEASE-VERSION']},
        install_requires=install_requires,
        )


if __name__ == "__main__":
    main()
